import { Module, MutationTree, ActionTree, GetterTree } from 'vuex';
import { StateInterface } from 'src/store';
import { axios } from 'boot/axios';
import { LoginData } from 'src/plugins/user/models';
import { AxiosResponse } from 'axios';
import { LocalStorage, Loading } from 'quasar';
import { Router } from 'src/router';

export interface UserStateInterface extends LoginResponse {
  loginLoading: boolean;
}

export interface LoginResponse {
  permissions: string[];
  session: Session;
}

const state: UserStateInterface = {
  permissions: [],
  session: {
    browser: '',
    expires: new Date(),
    lifetime: -1,
    platform: '',
    token: '',
    user: {
      display_name: '',
      firstname: '',
      lastname: '',
      mail: '',
      roles: [],
      userid: ''
    }
  },
  loginLoading: false
};

const mutations: MutationTree<UserStateInterface> = {
  setPermissions(state, data: []) {
    state.permissions = data;
  },
  setSession(state, data: Session) {
    state.session = data;
  },
  setLoginLoading(state, data: boolean) {
    state.loginLoading = data;
  },
  showState(state) {
    console.log(state);
  }
};

const actions: ActionTree<UserStateInterface, StateInterface> = {
  login({ commit }, data: LoginData) {
    commit('setLoginLoading', true);
    Loading.show({ message: 'Du wirst eingeloggt' });
    void axios
      .post('/auth', data)
      .then((response: AxiosResponse<LoginResponse>) => {
        commit('setPermissions', response.data.permissions);
        console.log('saved permisisons');
        commit('setSession', response.data.session);
        commit('showState');
        LocalStorage.set('permissions', response.data.permissions);
        LocalStorage.set('session', response.data.session);

        void Router.push({ name: 'user' });
      })
      .catch(error => {
        console.exception(error);
      })
      .finally(() => {
        commit('setLoginLoading', false);
        Loading.hide();
      });
  },
  logout({ commit }, token) {
    void axios.delete(`/auth/${token}`).then(() => {
      commit('setPermissions', []);
      commit('setToken', {
        browser: '',
        expires: '',
        lifetime: '',
        platform: '',
        token: ''
      });
    });
  },
  loadFromLocalStorage({ commit }) {
    console.log('load from store');
    let data = LocalStorage.getItem('permissions');
    commit('setPermissions', data ? data : []);
    data = LocalStorage.getItem('session');
    commit(
      'setSession',
      data
        ? data
        : {
            browser: '',
            expires: new Date(),
            lifetime: -1,
            platform: '',
            token: ''
          }
    );
    commit('showState');
  }
};

const getters: GetterTree<UserStateInterface, StateInterface> = {
  permissions({ permissions }) {
    return permissions;
  },
  session({ session }) {
    return session;
  },
  loginLoading({ loginLoading }) {
    return loginLoading;
  },
  displayName({ session: { user } }) {
    return `${user.firstname} ${user.lastname}`;
  }
};

const userStore: Module<UserStateInterface, StateInterface> = {
  namespaced: true,
  actions,
  getters,
  mutations,
  state
};

export default userStore;
